# -*- encoding: utf-8 -*-

import json

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from .helpers import get_compose, get_fixture

from compose_utils import get_listing, get_modular_packages


class ListingTest(unittest.TestCase):

    def test_get_listing(self):
        compose = get_compose("DP-1.0-20160315.t.0")
        with open(get_fixture("full-listing.json")) as fh:
            expected = json.load(fh)

        listing = get_listing(compose.rpms)

        self.assertEqual(listing, expected)

    def test_get_listing_filter_but_keep_all_variants(self):
        compose = get_compose("DP-1.0-20160315.t.0")
        with open(get_fixture("full-listing.json")) as fh:
            expected = json.load(fh)

        listing = get_listing(compose.rpms, variants=["Client", "Server"])

        self.assertEqual(listing, expected)

    def test_get_listing_filter_but_keep_all_arches(self):
        compose = get_compose("DP-1.0-20160315.t.0")
        with open(get_fixture("full-listing.json")) as fh:
            expected = json.load(fh)

        listing = get_listing(compose.rpms, arches=["x86_64", "i386", "s390x"])

        self.assertEqual(listing, expected)

    def test_get_listing_filter_trees(self):
        compose = get_compose("DP-1.0-20160315.t.0")
        with open(get_fixture("listing-server-x86_64.json")) as fh:
            expected = json.load(fh)

        listing = get_listing(compose.rpms, variants=["Server"], arches=["x86_64"])

        self.assertEqual(listing, expected)

    def test_get_listing_filter_pkg_arch(self):
        compose = get_compose("DP-1.0-20160315.t.0")
        with open(get_fixture("listing-multilib.json")) as fh:
            expected = json.load(fh)

        listing = get_listing(compose.rpms, pkg_arches=["i686"], arches=["x86_64"])

        self.assertEqual(listing, expected)

    def test_get_listing_skip_packages(self):
        compose = get_compose("DP-1.0-20160315.t.0")
        with open(get_fixture("filtered-listing.json")) as fh:
            expected = json.load(fh)

        self.maxDiff = None
        skip_nevras = {
            "Server": {
                "s390x": set(["dummy-basesystem-0:10.0-6.noarch"]),
            },
            "Client": {
                "x86_64": set(["Dummy-firefox-0:16.0.1-1.x86_64"]),
            },
        }

        listing = get_listing(compose.rpms, skip_nevras=skip_nevras)

        self.assertEqual(listing, expected)


class GetModularPackagesTest(unittest.TestCase):

    def test_load(self):
        compose = get_compose("DP-1.0-20160315.t.0")

        self.assertEqual(
            get_modular_packages(compose),
            {
                "Server": {
                    "aarch64": set([
                        "fish-0:3.0.0-1.module_f30+2710+8dbb4023.aarch64",
                        "fish-0:3.0.0-1.module_f30+2703+82e92e7e.aarch64",
                    ]),
                    "x86_64": set([
                        "perl-List-Compare-0:0.53-11.module_f30+2696+4b2456b0.noarch",
                        "dwm-user-0:6.1-8.module_f30+2696+4b2456b0.x86_64",
                        "dwm-0:6.1-8.module_f30+2696+4b2456b0.x86_64"
                    ]),
                }
            }
        )
